@@ -155,7 +155,7 @@ void QDeclarativeCategory::pluginReady()
 
 /*!
     \qmlproperty QPlaceCategory Category::category
-    \target Category::category
+    \keyword Category::category
 
     For details on how to use this property to interface between C++ and QML see
     "\l {Category - QPlaceCategory} {Interfaces between C++ and QML Code}".
