@@ -1,39 +1,34 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtLocation module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL3$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or later as published by the Free
+** Software Foundation and appearing in the file LICENSE.GPL included in
+** the packaging of this file. Please review the following information to
+** ensure the GNU General Public License version 2.0 requirements will be
+** met: http://www.gnu.org/licenses/gpl-2.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -56,7 +51,7 @@ QT_USE_NAMESPACE
     \inqmlmodule QtLocation
     \ingroup qml-QtLocation5-places
     \ingroup qml-QtLocation5-places-models
-    \since Qt Location 5.0
+    \since Qt Location 5.5
 
     \brief Provides access to search term suggestions.
 
@@ -92,7 +87,7 @@ QT_USE_NAMESPACE
     actual place search with \l PlaceSearchModel.
 
     \snippet declarative/places.qml QtQuick import
-    \snippet declarative/places.qml QtLocation import
+    \snippet declarative/maps.qml QtLocation import
     \codeline
     \snippet declarative/places.qml SearchSuggestionModel
 
@@ -154,7 +149,7 @@ QT_USE_NAMESPACE
 */
 
 /*!
-    \qmlmethod PlaceSearchSuggestionModel::update()
+    \qmlmethod void PlaceSearchSuggestionModel::update()
 
     Updates the model based on the provided query parameters.  The model will be populated with a
     list of search suggestions for the partial \l searchTerm and \l searchArea.  If the \l plugin
@@ -194,7 +189,7 @@ QT_USE_NAMESPACE
 */
 
 /*!
-    \qmlmethod PlaceSearchSuggestionModel::cancel()
+    \qmlmethod void PlaceSearchSuggestionModel::cancel()
 
     Cancels an ongoing search suggestion operation immediately and sets the model
     status to PlaceSearchSuggestionModel.Ready.  The model retains any search
@@ -206,7 +201,7 @@ QT_USE_NAMESPACE
 */
 
 /*!
-    \qmlmethod PlaceSearchSuggestionModel::reset()
+    \qmlmethod void PlaceSearchSuggestionModel::reset()
 
     Resets the model.  All search suggestions are cleared, any outstanding requests are aborted and
     possible errors are cleared.  Model status will be set to PlaceSearchSuggestionModel.Null.
@@ -216,7 +211,7 @@ QT_USE_NAMESPACE
 /*!
     \qmlmethod string QtLocation::PlaceSearchSuggestionModel::errorString() const
 
-    This read-only property holds the textual presentation of latest search suggestion model error.
+    This read-only property holds the textual presentation of the latest search suggestion model error.
     If no error has occurred, or if the model was cleared, an empty string is returned.
 
     An empty string may also be returned if an error occurred which has no associated
