@@ -37,6 +37,17 @@
 #ifndef QDECLARATIVECIRCLEMAPITEM_H
 #define QDECLARATIVECIRCLEMAPITEM_H
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  It exists purely as an
+// implementation detail.  This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
+
 #include "qdeclarativegeomapitembase_p.h"
 #include "qdeclarativepolylinemapitem_p.h"
 #include "qdeclarativepolygonmapitem_p.h"
@@ -45,8 +56,6 @@
 
 QT_BEGIN_NAMESPACE
 
-class QDeclarativeGeoMapQuickItem;
-
 class QGeoMapCircleGeometry : public QGeoMapPolygonGeometry
 {
 public:
@@ -90,15 +99,15 @@ Q_SIGNALS:
 
 protected:
     void geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry) Q_DECL_OVERRIDE;
+    void updatePolish() Q_DECL_OVERRIDE;
 
 protected Q_SLOTS:
-    virtual void updateMapItem() Q_DECL_OVERRIDE;
-    void updateMapItemAssumeDirty();
+    void markSourceDirtyAndUpdate();
     virtual void afterViewportChanged(const QGeoMapViewportChangeEvent &event) Q_DECL_OVERRIDE;
 
 private:
     bool preserveCircleGeometry(QList<QGeoCoordinate> &path, const QGeoCoordinate &center,
-                                qreal distance, QGeoCoordinate &leftBoundCoord);
+                                qreal distance);
     void updateCirclePathForRendering(QList<QGeoCoordinate> &path, const QGeoCoordinate &center,
                                       qreal distance);
 
@@ -107,6 +116,7 @@ private:
     QDeclarativeMapLineProperties border_;
     QColor color_;
     qreal radius_;
+    QGeoCoordinate geoLeftBound_;
     QList<QGeoCoordinate> circlePath_;
     bool dirtyMaterial_;
     QGeoMapCircleGeometry geometry_;
