@@ -1,39 +1,34 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtLocation module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL3$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or later as published by the Free
+** Software Foundation and appearing in the file LICENSE.GPL included in
+** the packaging of this file. Please review the following information to
+** ensure the GNU General Public License version 2.0 requirements will be
+** met: http://www.gnu.org/licenses/gpl-2.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -55,7 +50,7 @@ QT_BEGIN_NAMESPACE
     \instantiates QDeclarativeGeocodeModel
     \inqmlmodule QtLocation
     \ingroup qml-QtLocation5-geocoding
-    \since Qt Location 5.0
+    \since Qt Location 5.5
 
     \brief The GeocodeModel type provides support for searching operations
            related to geographic information.
@@ -108,6 +103,15 @@ QT_BEGIN_NAMESPACE
     \endcode
 */
 
+/*!
+    \qmlsignal QtLocation::GeocodeModel::locationsChanged()
+
+    This signal is emitted when locations in the model have changed.
+
+    \sa count
+*/
+
+
 QDeclarativeGeocodeModel::QDeclarativeGeocodeModel(QObject *parent)
 :   QAbstractListModel(parent), autoUpdate_(false), complete_(false), reply_(0), plugin_(0),
     status_(QDeclarativeGeocodeModel::Null), error_(QDeclarativeGeocodeModel::NoError),
@@ -134,7 +138,7 @@ void QDeclarativeGeocodeModel::componentComplete()
 }
 
 /*!
-    \qmlmethod QtLocation::GeocodeModel::update()
+    \qmlmethod void QtLocation::GeocodeModel::update()
 
     Instructs the GeocodeModel to update its data. This is most useful
     when \l autoUpdate is disabled, to force a refresh when the query
@@ -146,7 +150,7 @@ void QDeclarativeGeocodeModel::update()
         return;
 
     if (!plugin_) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, GEOCODE_PLUGIN_NOT_SET);
+        setError(EngineNotSetError, tr("Cannot geocode, plugin not set."));
         return;
     }
 
@@ -156,17 +160,16 @@ void QDeclarativeGeocodeModel::update()
 
     QGeoCodingManager *geocodingManager = serviceProvider->geocodingManager();
     if (!geocodingManager) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, GEOCODE_MGR_NOT_SET);
+        setError(EngineNotSetError, tr("Cannot geocode, geocode manager not set."));
         return;
     }
     if (!coordinate_.isValid() && (!address_ || address_->address().isEmpty()) &&
         (searchString_.isEmpty())) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, GEOCODE_QUERY_NOT_SET);
+        setError(ParseError, tr("Cannot geocode, valid query not set."));
         return;
     }
     abortRequest(); // abort possible previous requests
-    setErrorString("");   // clear previous error string
-    setError(NoError);
+    setError(NoError, QString());
 
     if (coordinate_.isValid()) {
         setStatus(QDeclarativeGeocodeModel::Loading);
@@ -287,10 +290,31 @@ void QDeclarativeGeocodeModel::pluginReady()
 {
     QGeoServiceProvider *serviceProvider = plugin_->sharedGeoServiceProvider();
     QGeoCodingManager *geocodingManager = serviceProvider->geocodingManager();
-    if (!geocodingManager || serviceProvider->error() != QGeoServiceProvider::NoError) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, PLUGIN_DOESNOT_SUPPORT_GEOCODING).arg(serviceProvider->errorString());
+
+    if (serviceProvider->error() != QGeoServiceProvider::NoError) {
+        QDeclarativeGeocodeModel::GeocodeError newError = UnknownError;
+        switch (serviceProvider->error()) {
+        case QGeoServiceProvider::NotSupportedError:
+            newError = EngineNotSetError; break;
+        case QGeoServiceProvider::UnknownParameterError:
+            newError = UnknownParameterError; break;
+        case QGeoServiceProvider::MissingRequiredParameterError:
+            newError = MissingRequiredParameterError; break;
+        case QGeoServiceProvider::ConnectionError:
+            newError = CommunicationError; break;
+        default:
+            break;
+        }
+
+        setError(newError, serviceProvider->errorString());
         return;
     }
+
+    if (!geocodingManager) {
+        setError(EngineNotSetError,tr("Plugin does not support (reverse) geocoding."));
+        return;
+    }
+
     connect(geocodingManager, SIGNAL(finished(QGeoCodeReply*)),
             this, SLOT(geocodeFinished(QGeoCodeReply*)));
     connect(geocodingManager, SIGNAL(error(QGeoCodeReply*,QGeoCodeReply::Error,QString)),
@@ -357,8 +381,7 @@ void QDeclarativeGeocodeModel::geocodeFinished(QGeoCodeReply *reply)
         return;
     int oldCount = declarativeLocations_.count();
     setLocations(reply->locations());
-    setErrorString("");
-    setError(NoError);
+    setError(NoError, QString());
     setStatus(QDeclarativeGeocodeModel::Ready);
     reply->deleteLater();
     reply_ = 0;
@@ -384,8 +407,7 @@ void QDeclarativeGeocodeModel::geocodeError(QGeoCodeReply *reply,
         emit locationsChanged();
         emit countChanged();
     }
-    setErrorString(errorString);
-    setError(static_cast<QDeclarativeGeocodeModel::GeocodeError>(error));
+    setError(static_cast<QDeclarativeGeocodeModel::GeocodeError>(error), errorString);
     setStatus(QDeclarativeGeocodeModel::Error);
     reply->deleteLater();
     reply_ = 0;
@@ -423,13 +445,17 @@ void QDeclarativeGeocodeModel::setStatus(QDeclarativeGeocodeModel::Status status
     This read-only property holds the latest error value of the geocoding request.
 
     \list
-    \li GeocodeModel.NoError - No error has occurred
-    \li GeocodeModel.EngineNotSetError - The plugin/service provider used does not support (reverse) geocoding
-    \li GeocodeModel.CommunicationError - An error occurred while communicating with the service provider
-    \li GeocodeModel.ParseError - The response from the service provider was in an unrecognizable format
-    \li GeocodeModel.UnsupportedOptionError - The requested operation or one of the options for the operation are not supported by the service provider.
-    \li GeocodeModel.CombinationError - An error occurred while results where being combined from multiple sources
-    \li GeocodeModel.UnknownError - An error occurred which does not fit into any of the other categories
+    \li GeocodeModel.NoError - No error has occurred.
+    \li GeocodeModel.CombinationError - An error occurred while results where being combined from multiple sources.
+    \li GeocodeModel.CommunicationError - An error occurred while communicating with the service provider.
+    \li GeocodeModel.EngineNotSetError - The model's plugin property was not set or there is no geocoding manager associated with the plugin.
+    \li GeocodeModel.MissingRequiredParameterError - A required parameter was not specified.
+    \li GeocodeModel.ParseError - The response from the service provider was in an unrecognizable format.
+    \li GeocodeModel.UnknownError - An error occurred which does not fit into any of the other categories.
+    \li GeocodeModel.UnknownParameterError - The plugin did not recognize one of the parameters it was given.
+    \li GeocodeModel.UnsupportedOptionError - The requested operation is not supported by the geocoding provider.
+                                              This may happen when the loaded engine does not support a particular geocoding request
+                                              such as reverse geocoding.
     \endlist
 */
 
@@ -438,18 +464,19 @@ QDeclarativeGeocodeModel::GeocodeError QDeclarativeGeocodeModel::error() const
     return error_;
 }
 
-void QDeclarativeGeocodeModel::setError(GeocodeError error)
+void QDeclarativeGeocodeModel::setError(GeocodeError error, const QString &errorString)
 {
-    if (error_ == error)
+    if (error_ == error && errorString_ == errorString)
         return;
     error_ = error;
+    errorString_ = errorString;
     emit errorChanged();
 }
 
 /*!
     \qmlproperty string QtLocation::GeocodeModel::errorString
 
-    This read-only property holds the textual presentation of latest geocoding error.
+    This read-only property holds the textual presentation of the latest geocoding error.
     If no error has occurred or the model has been reset, an empty string is returned.
 
     An empty string may also be returned if an error occurred which has no associated
@@ -461,14 +488,6 @@ QString QDeclarativeGeocodeModel::errorString() const
     return errorString_;
 }
 
-void QDeclarativeGeocodeModel::setErrorString(const QString &error)
-{
-    if (errorString_ == error)
-        return;
-    errorString_ = error;
-    emit errorStringChanged();
-}
-
 /*!
     \internal
 */
@@ -510,7 +529,7 @@ int QDeclarativeGeocodeModel::count() const
 QDeclarativeGeoLocation *QDeclarativeGeocodeModel::get(int index)
 {
     if (index < 0 || index >= declarativeLocations_.count()) {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, INDEX_OUT_OF_RANGE).arg(index);
+        qmlInfo(this) << QStringLiteral("Index '%1' out of range").arg(index);
         return 0;
     }
     return declarativeLocations_.at(index);
@@ -570,7 +589,7 @@ void QDeclarativeGeocodeModel::setOffset(int offset)
 }
 
 /*!
-    \qmlmethod QtLocation::GeocodeModel::reset()
+    \qmlmethod void QtLocation::GeocodeModel::reset()
 
     Resets the model. All location data is cleared, any outstanding requests
     are aborted and possible errors are cleared. Model status will be set
@@ -587,13 +606,12 @@ void QDeclarativeGeocodeModel::reset()
     endResetModel();
 
     abortRequest();
-    setErrorString(QString());
-    setError(NoError);
+    setError(NoError, QString());
     setStatus(QDeclarativeGeocodeModel::Null);
 }
 
 /*!
-    \qmlmethod QtLocation::GeocodeModel::cancel()
+    \qmlmethod void QtLocation::GeocodeModel::cancel()
 
     Cancels any outstanding requests and clears errors.  Model status will be set to either
     GeocodeModel.Null or GeocodeModel.Ready.
@@ -601,8 +619,7 @@ void QDeclarativeGeocodeModel::reset()
 void QDeclarativeGeocodeModel::cancel()
 {
     abortRequest();
-    setErrorString(QString());
-    setError(NoError);
+    setError(NoError, QString());
     setStatus(declarativeLocations_.isEmpty() ? Null : Ready);
 }
 
@@ -662,11 +679,13 @@ void QDeclarativeGeocodeModel::setQuery(const QVariant &query)
             connect(address_, SIGNAL(streetChanged()), this, SLOT(queryContentChanged()));
             connect(address_, SIGNAL(postalCodeChanged()), this, SLOT(queryContentChanged()));
         } else {
-            qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, UNSUPPORTED_QUERY_TYPE);
+            qmlInfo(this) << QStringLiteral("Unsupported query type for geocode model ")
+                          << QStringLiteral("(coordinate, string and Address supported).");
             return;
         }
     } else {
-        qmlInfo(this) << QCoreApplication::translate(CONTEXT_NAME, UNSUPPORTED_QUERY_TYPE);
+        qmlInfo(this) << QStringLiteral("Unsupported query type for geocode model ")
+                      << QStringLiteral("(coordinate, string and Address supported).");
         return;
     }
 
