@@ -39,7 +39,6 @@
 #include "qdeclarativegeomapcopyrightsnotice_p.h"
 #include "qdeclarativegeoserviceprovider_p.h"
 #include "qdeclarativegeomaptype_p.h"
-#include "qgeomapcontroller_p.h"
 #include "qgeomappingmanager_p.h"
 #include "qgeocameracapabilities_p.h"
 #include "qgeomap_p.h"
@@ -161,16 +160,16 @@ QDeclarativeGeoMap::QDeclarativeGeoMap(QQuickItem *parent)
         m_plugin(0),
         m_serviceProvider(0),
         m_mappingManager(0),
-        m_center(51.5073,-0.1277), //London city center
         m_activeMapType(0),
         m_gestureArea(new QQuickGeoMapGestureArea(this)),
         m_map(0),
         m_error(QGeoServiceProvider::NoError),
-        m_zoomLevel(8.0),
-        m_componentCompleted(false),
-        m_mappingManagerInitialized(false),
         m_color(QColor::fromRgbF(0.9, 0.9, 0.9)),
+        m_componentCompleted(false),
         m_pendingFitViewport(false),
+        m_copyrightsVisible(true),
+        m_maximumViewportLatitude(0.0),
+        m_initialized(false),
         m_validRegion(false)
 {
     setAcceptHoverEvents(false);
@@ -183,6 +182,8 @@ QDeclarativeGeoMap::QDeclarativeGeoMap(QQuickItem *parent)
     m_activeMapType = new QDeclarativeGeoMapType(QGeoMapType(QGeoMapType::NoMap,
                                                              tr("No Map"),
                                                              tr("No Map"), false, false, 0), this);
+    m_cameraData.setCenter(QGeoCoordinate(51.5073,-0.1277)); //London city center
+    m_cameraData.setZoomLevel(8.0);
 }
 
 QDeclarativeGeoMap::~QDeclarativeGeoMap()
@@ -205,7 +206,7 @@ QDeclarativeGeoMap::~QDeclarativeGeoMap()
 */
 void QDeclarativeGeoMap::onMapChildrenChanged()
 {
-    if (!m_componentCompleted || !m_mappingManagerInitialized)
+    if (!m_componentCompleted || !m_map)
         return;
 
     int maxChildZ = 0;
@@ -239,6 +240,10 @@ void QDeclarativeGeoMap::onMapChildrenChanged()
                     copyrights, SLOT(copyrightsChanged(QString)));
             connect(copyrights, SIGNAL(linkActivated(QString)),
                     this, SIGNAL(copyrightLinkActivated(QString)));
+
+            // set visibility of copyright notice
+            copyrights->setCopyrightsVisible(m_copyrightsVisible);
+
         } else {
             // just re-set its parent.
             copyrights->setParent(this);
@@ -296,6 +301,31 @@ void QDeclarativeGeoMap::setError(QGeoServiceProvider::Error error, const QStrin
     emit errorChanged();
 }
 
+void QDeclarativeGeoMap::initialize()
+{
+    // try to keep center change signal in the end
+    bool centerHasChanged = false;
+
+    setMinimumZoomLevel(m_map->minimumZoomForMapSize(width(), height()));
+
+    // set latitude bundary check
+    m_maximumViewportLatitude = m_map->maximumLatitudeForZoom(m_cameraData.zoomLevel());
+    QGeoCoordinate center = m_cameraData.center();
+    center.setLatitude(qBound(-m_maximumViewportLatitude, center.latitude(), m_maximumViewportLatitude));
+
+    if (center != m_cameraData.center()) {
+        centerHasChanged = true;
+        m_cameraData.setCenter(center);
+    }
+
+    m_map->setCameraData(m_cameraData);
+
+    m_initialized = true;
+
+    if (centerHasChanged)
+        emit centerChanged(m_cameraData.center());
+}
+
 /*!
     \internal
 */
@@ -494,49 +524,13 @@ void QDeclarativeGeoMap::setPlugin(QDeclarativeGeoServiceProvider *plugin)
 */
 void QDeclarativeGeoMap::mappingManagerInitialized()
 {
-    m_mappingManagerInitialized = true;
-
     m_map = m_mappingManager->createMap(this);
+
+    if (!m_map)
+        return;
+
     m_gestureArea->setMap(m_map);
 
-    // once mappingManagerInitilized_ is set zoomLevel() returns the default initialised
-    // zoom level of the map controller. Overwrite it here to whatever the user chose.
-    m_map->mapController()->setZoom(m_zoomLevel);
-
-    //The zoom level limits are only restricted by the plugins values, if the user has set a more
-    //strict zoom level limit before initialization nothing is done here.
-    if (m_mappingManager->cameraCapabilities().minimumZoomLevel() > m_gestureArea->minimumZoomLevel())
-        setMinimumZoomLevel(m_mappingManager->cameraCapabilities().minimumZoomLevel());
-
-    if (m_gestureArea->maximumZoomLevel() < 0
-            || m_mappingManager->cameraCapabilities().maximumZoomLevel() < m_gestureArea->maximumZoomLevel())
-        setMaximumZoomLevel(m_mappingManager->cameraCapabilities().maximumZoomLevel());
-
-    m_map->setActiveMapType(QGeoMapType());
-
-    m_copyrights = new QDeclarativeGeoMapCopyrightNotice(this);
-    connect(m_map, SIGNAL(copyrightsChanged(QImage)),
-            m_copyrights.data(), SLOT(copyrightsChanged(QImage)));
-    connect(m_map, SIGNAL(copyrightsChanged(QString)),
-            m_copyrights.data(), SLOT(copyrightsChanged(QString)));
-    connect(m_copyrights.data(), SIGNAL(linkActivated(QString)),
-            this, SIGNAL(copyrightLinkActivated(QString)));
-
-    connect(m_map,
-            SIGNAL(updateRequired()),
-            this,
-            SLOT(update()));
-    connect(m_map->mapController(),
-            SIGNAL(centerChanged(QGeoCoordinate)),
-            this,
-            SLOT(mapCenterChanged(QGeoCoordinate)));
-    connect(m_map->mapController(),
-            SIGNAL(zoomChanged(qreal)),
-            this,
-            SLOT(mapZoomLevelChanged(qreal)));
-
-    m_map->mapController()->setCenter(m_center);
-
     QList<QGeoMapType> types = m_mappingManager->supportedMapTypes();
     for (int i = 0; i < types.size(); ++i) {
         QDeclarativeGeoMapType *type = new QDeclarativeGeoMapType(types[i], this);
@@ -547,13 +541,44 @@ void QDeclarativeGeoMap::mappingManagerInitialized()
         QDeclarativeGeoMapType *type = m_supportedMapTypes.at(0);
         m_activeMapType = type;
         m_map->setActiveMapType(type->mapType());
+    } else {
+        m_map->setActiveMapType(m_activeMapType->mapType());
     }
 
-    // Map tiles are built in this call
-    m_map->resize(width(), height());
+    //The zoom level limits are only restricted by the plugins values, if the user has set a more
+    //strict zoom level limit before initialization nothing is done here.
+    //minimum zoom level might be changed to limit gray bundaries
+
+    if (m_gestureArea->maximumZoomLevel() < 0
+            || m_mappingManager->cameraCapabilities().maximumZoomLevel() < m_gestureArea->maximumZoomLevel())
+        setMaximumZoomLevel(m_mappingManager->cameraCapabilities().maximumZoomLevel());
+
+    if (m_mappingManager->cameraCapabilities().minimumZoomLevel() > m_gestureArea->minimumZoomLevel())
+        setMinimumZoomLevel(m_mappingManager->cameraCapabilities().minimumZoomLevel());
+
+
+    // Map tiles are built in this call. m_map->minimumZoom() becomes operational
+    // after this has been called at least once, after creation.
+
+    if (!m_initialized && width() > 0 && height() > 0) {
+        m_map->setSize(QSize(width(), height()));
+        initialize();
+    }
+
+    m_copyrights = new QDeclarativeGeoMapCopyrightNotice(this);
+    connect(m_map, SIGNAL(copyrightsChanged(QImage)),
+            m_copyrights.data(), SLOT(copyrightsChanged(QImage)));
+    connect(m_map, SIGNAL(copyrightsChanged(QString)),
+            m_copyrights.data(), SLOT(copyrightsChanged(QString)));
+    connect(m_copyrights.data(), SIGNAL(linkActivated(QString)),
+            this, SIGNAL(copyrightLinkActivated(QString)));
+    connect(m_map, &QGeoMap::sgNodeChanged, this, &QQuickItem::update);
+
+    // set visibility of copyright notice
+    m_copyrights->setCopyrightsVisible(m_copyrightsVisible);
+
     // This prefetches a buffer around the map
     m_map->prefetchData();
-    m_map->update();
 
     connect(m_mappingManager, SIGNAL(supportedMapTypesChanged()), this, SLOT(onSupportedMapTypesChanged()));
     emit minimumZoomLevelChanged();
@@ -581,17 +606,27 @@ QDeclarativeGeoServiceProvider *QDeclarativeGeoMap::plugin() const
     \internal
     Sets the gesture areas minimum zoom level. If the camera capabilities
     has been set this method honors the boundaries set by it.
+    The minimum zoom level will also have a lower bound dependent on the size
+    of the canvas, effectively preventing to display out of bounds areas.
 */
 void QDeclarativeGeoMap::setMinimumZoomLevel(qreal minimumZoomLevel)
 {
-    if (m_gestureArea && minimumZoomLevel >= 0) {
+
+    if (minimumZoomLevel >= 0) {
         qreal oldMinimumZoomLevel = this->minimumZoomLevel();
-        if (m_mappingManagerInitialized
-                && minimumZoomLevel < m_mappingManager->cameraCapabilities().minimumZoomLevel()) {
-            minimumZoomLevel = m_mappingManager->cameraCapabilities().minimumZoomLevel();
+
+        if (m_map) {
+            minimumZoomLevel = qBound(qreal(m_map->cameraCapabilities().minimumZoomLevel()), minimumZoomLevel, maximumZoomLevel());
+            double minimumViewportZoomLevel = m_map->minimumZoomForMapSize(width(),height());
+            if (minimumZoomLevel < minimumViewportZoomLevel)
+                minimumZoomLevel = minimumViewportZoomLevel;
         }
+
         m_gestureArea->setMinimumZoomLevel(minimumZoomLevel);
-        setZoomLevel(qBound<qreal>(minimumZoomLevel, zoomLevel(), maximumZoomLevel()));
+
+        if (zoomLevel() < minimumZoomLevel)
+            setZoomLevel(minimumZoomLevel);
+
         if (oldMinimumZoomLevel != minimumZoomLevel)
             emit minimumZoomLevelChanged();
     }
@@ -602,7 +637,12 @@ void QDeclarativeGeoMap::setMinimumZoomLevel(qreal minimumZoomLevel)
 
     This property holds the minimum valid zoom level for the map.
 
-    The minimum zoom level is defined by the \l plugin used.
+    The minimum zoom level defined by the \l plugin used is a lower bound for
+    this property. However, the returned value is also canvas-size-dependent, and
+    can be higher than the user-specified value, or than the minimum zoom level
+    defined by the plugin used, to prevent the map from being smaller than the
+    viewport in either dimension.
+
     If a plugin supporting mapping is not set, -1.0 is returned.
 */
 
@@ -610,8 +650,8 @@ qreal QDeclarativeGeoMap::minimumZoomLevel() const
 {
     if (m_gestureArea->minimumZoomLevel() != -1)
         return m_gestureArea->minimumZoomLevel();
-    else if (m_mappingManager && m_mappingManagerInitialized)
-        return m_mappingManager->cameraCapabilities().minimumZoomLevel();
+    else if (m_map)
+        return m_map->cameraCapabilities().minimumZoomLevel();
     else
         return -1.0;
 }
@@ -623,14 +663,17 @@ qreal QDeclarativeGeoMap::minimumZoomLevel() const
 */
 void QDeclarativeGeoMap::setMaximumZoomLevel(qreal maximumZoomLevel)
 {
-    if (m_gestureArea && maximumZoomLevel >= 0) {
+    if (maximumZoomLevel >= 0) {
         qreal oldMaximumZoomLevel = this->maximumZoomLevel();
-        if (m_mappingManagerInitialized
-                && maximumZoomLevel > m_mappingManager->cameraCapabilities().maximumZoomLevel()) {
-            maximumZoomLevel = m_mappingManager->cameraCapabilities().maximumZoomLevel();
-        }
+
+        if (m_map)
+            maximumZoomLevel = qBound(minimumZoomLevel(), maximumZoomLevel, qreal(m_map->cameraCapabilities().maximumZoomLevel()));
+
         m_gestureArea->setMaximumZoomLevel(maximumZoomLevel);
-        setZoomLevel(qBound<qreal>(minimumZoomLevel(), zoomLevel(), maximumZoomLevel));
+
+        if (zoomLevel() > maximumZoomLevel)
+            setZoomLevel(maximumZoomLevel);
+
         if (oldMaximumZoomLevel != maximumZoomLevel)
             emit maximumZoomLevelChanged();
     }
@@ -649,8 +692,8 @@ qreal QDeclarativeGeoMap::maximumZoomLevel() const
 {
     if (m_gestureArea->maximumZoomLevel() != -1)
         return m_gestureArea->maximumZoomLevel();
-    else if (m_mappingManager && m_mappingManagerInitialized)
-        return m_mappingManager->cameraCapabilities().maximumZoomLevel();
+    else if (m_map)
+        return m_map->cameraCapabilities().maximumZoomLevel();
     else
         return -1.0;
 }
@@ -665,27 +708,36 @@ qreal QDeclarativeGeoMap::maximumZoomLevel() const
 */
 void QDeclarativeGeoMap::setZoomLevel(qreal zoomLevel)
 {
-    if (m_zoomLevel == zoomLevel || zoomLevel < 0)
+    if (m_cameraData.zoomLevel() == zoomLevel || zoomLevel < 0)
         return;
 
-    if ((zoomLevel < minimumZoomLevel()
-         || (maximumZoomLevel() >= 0 && zoomLevel > maximumZoomLevel())))
-        return;
+    //small optiomatization to avoid double setCameraData
+    bool centerHasChanged = false;
+
+    if (m_initialized) {
+        m_cameraData.setZoomLevel(qBound(minimumZoomLevel(), zoomLevel, maximumZoomLevel()));
+        m_maximumViewportLatitude = m_map->maximumLatitudeForZoom(m_cameraData.zoomLevel());
+        QGeoCoordinate coord = m_cameraData.center();
+        coord.setLatitude(qBound(-m_maximumViewportLatitude, coord.latitude(), m_maximumViewportLatitude));
+        if (coord != m_cameraData.center()) {
+            centerHasChanged = true;
+            m_cameraData.setCenter(coord);
+        }
+        m_map->setCameraData(m_cameraData);
+    } else {
+        m_cameraData.setZoomLevel(zoomLevel);
+    }
 
-    m_zoomLevel = zoomLevel;
     m_validRegion = false;
 
-    if (m_mappingManagerInitialized)
-        m_map->mapController()->setZoom(m_zoomLevel);
-    emit zoomLevelChanged(zoomLevel);
+    if (centerHasChanged)
+        emit centerChanged(m_cameraData.center());
+    emit zoomLevelChanged(m_cameraData.zoomLevel());
 }
 
 qreal QDeclarativeGeoMap::zoomLevel() const
 {
-    if (m_mappingManagerInitialized)
-        return m_map->mapController()->zoom();
-    else
-        return m_zoomLevel;
+    return m_cameraData.zoomLevel();
 }
 
 /*!
@@ -698,29 +750,28 @@ qreal QDeclarativeGeoMap::zoomLevel() const
 */
 void QDeclarativeGeoMap::setCenter(const QGeoCoordinate &center)
 {
-    if (!m_mappingManagerInitialized && center == m_center)
+    if (center == m_cameraData.center())
         return;
 
     if (!center.isValid())
         return;
 
-    m_center = center;
-    m_validRegion = false;
-
-    if (m_center.isValid() && m_mappingManagerInitialized) {
-        m_map->mapController()->setCenter(m_center);
-        update();
+    if (m_initialized) {
+        QGeoCoordinate coord(center);
+        coord.setLatitude(qBound(-m_maximumViewportLatitude, center.latitude(), m_maximumViewportLatitude));
+        m_cameraData.setCenter(coord);
+        m_map->setCameraData(m_cameraData);
     } else {
-        emit centerChanged(m_center);
+        m_cameraData.setCenter(center);
     }
+
+    m_validRegion = false;
+    emit centerChanged(m_cameraData.center());
 }
 
 QGeoCoordinate QDeclarativeGeoMap::center() const
 {
-    if (m_mappingManagerInitialized)
-        return m_map->mapController()->center();
-    else
-        return m_center;
+    return m_cameraData.center();
 }
 
 
@@ -773,6 +824,35 @@ QGeoShape QDeclarativeGeoMap::visibleRegion() const
     return QGeoRectangle(tl, br);
 }
 
+/*!
+    \qmlproperty bool QtLocation::Map::copyrightsVisible
+
+    This property holds the visibility of the copyrights notice. The notice is usually
+    displayed in the bottom left corner. By default, this property is set to \c true.
+
+    \note Many map providers require the notice to be visible as part of the terms and conditions.
+    Please consult the relevant provider documentation before turning this notice off.
+
+    \since 5.7
+*/
+void QDeclarativeGeoMap::setCopyrightsVisible(bool visible)
+{
+    if (m_copyrightsVisible == visible)
+        return;
+
+    if (!m_copyrights.isNull())
+        m_copyrights->setCopyrightsVisible(visible);
+
+    m_copyrightsVisible = visible;
+    emit copyrightsVisibleChanged(visible);
+}
+
+bool QDeclarativeGeoMap::copyrightsVisible() const
+{
+    return m_copyrightsVisible;
+}
+
+
 
 /*!
     \qmlproperty color QtLocation::Map::color
@@ -810,9 +890,6 @@ void QDeclarativeGeoMap::fitViewportToGeoShape()
         QGeoRectangle rect = m_region;
         topLeft = rect.topLeft();
         bottomRight = rect.bottomRight();
-        if (bottomRight.longitude() < topLeft.longitude())
-            bottomRight.setLongitude(bottomRight.longitude() + 360.0);
-
         break;
     }
     case QGeoShape::CircleType:
@@ -892,29 +969,12 @@ void QDeclarativeGeoMap::fitViewportToGeoShape()
                             bboxHeight / (height() - margins));
     // fixme: use log2 with c++11
     zoomRatio = std::log(zoomRatio) / std::log(2.0);
-    double newZoom = qMax<double>(minimumZoomLevel(), m_map->mapController()->zoom()
+    double newZoom = qMax(minimumZoomLevel(), zoomLevel()
                           - zoomRatio);
     setZoomLevel(newZoom);
     m_validRegion = true;
 }
 
-/*!
-    \internal
-*/
-void QDeclarativeGeoMap::mapZoomLevelChanged(qreal zoom)
-{
-    if (zoom == m_zoomLevel)
-        return;
-    m_zoomLevel = zoom;
-    m_validRegion = false;
-    emit zoomLevelChanged(m_zoomLevel);
-}
-
-void QDeclarativeGeoMap::mapCenterChanged(const QGeoCoordinate &center)
-{
-    m_validRegion = false;
-    emit centerChanged(center);
-}
 
 /*!
     \qmlproperty list<MapType> QtLocation::Map::supportedMapTypes
@@ -973,9 +1033,14 @@ QPointF QDeclarativeGeoMap::fromCoordinate(const QGeoCoordinate &coordinate, boo
 */
 void QDeclarativeGeoMap::pan(int dx, int dy)
 {
-    if (!m_mappingManagerInitialized)
+    if (!m_map)
         return;
-    m_map->mapController()->pan(dx, dy);
+    if (dx == 0 && dy == 0)
+        return;
+    QGeoCoordinate coord = m_map->itemPositionToCoordinate(
+                                QDoubleVector2D(m_map->width() / 2 + dx,
+                                        m_map->height() / 2 + dy));
+    setCenter(coord);
 }
 
 
@@ -986,7 +1051,7 @@ void QDeclarativeGeoMap::pan(int dx, int dy)
 */
 void QDeclarativeGeoMap::prefetchData()
 {
-    if (!m_mappingManagerInitialized)
+    if (!m_map)
         return;
     m_map->prefetchData();
 }
@@ -1250,12 +1315,17 @@ QDeclarativeGeoMapType * QDeclarativeGeoMap::activeMapType() const
 void QDeclarativeGeoMap::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     m_gestureArea->setSize(newGeometry.size());
+    QQuickItem::geometryChanged(newGeometry, oldGeometry);
 
-    if (!m_mappingManagerInitialized)
+    if (!m_map || !newGeometry.size().isValid())
         return;
 
-    m_map->resize(newGeometry.width(), newGeometry.height());
-    QQuickItem::geometryChanged(newGeometry, oldGeometry);
+    m_map->setSize(newGeometry.size().toSize());
+
+    if (!m_initialized)
+        initialize();
+    else
+        setMinimumZoomLevel(m_map->minimumZoomForMapSize(newGeometry.width(), newGeometry.height()));
 
     /*!
         The fitViewportTo*() functions depend on a valid map geometry.
@@ -1282,8 +1352,6 @@ void QDeclarativeGeoMap::geometryChanged(const QRectF &newGeometry, const QRectF
 */
 void QDeclarativeGeoMap::fitViewportToMapItems()
 {
-    if (!m_mappingManagerInitialized)
-        return;
     fitViewportToMapItemsRefine(true);
 }
 
@@ -1292,6 +1360,9 @@ void QDeclarativeGeoMap::fitViewportToMapItems()
 */
 void QDeclarativeGeoMap::fitViewportToMapItemsRefine(bool refine)
 {
+    if (!m_map)
+        return;
+
     if (m_mapItems.size() == 0)
         return;
 
@@ -1373,7 +1444,7 @@ void QDeclarativeGeoMap::fitViewportToMapItemsRefine(bool refine)
         zoomRatio = bboxHeight / height();
 
     qreal newZoom = std::log10(zoomRatio) / std::log10(0.5);
-    newZoom = std::floor(qMax(minimumZoomLevel(), (m_map->mapController()->zoom() + newZoom)));
+    newZoom = std::floor(qMax(minimumZoomLevel(), (zoomLevel() + newZoom)));
     setProperty("zoomLevel", QVariant::fromValue(newZoom));
 
     // as map quick items retain the same screen size after the camera zooms in/out
