@@ -51,8 +51,7 @@
 #include "qdoublevector2d_p.h"
 
 /* poly2tri triangulator includes */
-#include "../../3rdparty/poly2tri/common/shapes.h"
-#include "../../3rdparty/poly2tri/sweep/cdt.h"
+#include "../../3rdparty/clip2tri/clip2tri.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -275,44 +274,55 @@ void QGeoMapPolygonGeometry::updateScreenPoints(const QGeoMap &map)
 
     screenOutline_ = ppi;
 
-    std::vector<p2t::Point*> curPts;
+#if 1
+    std::vector<std::vector<c2t::Point>> clipperPoints;
+    clipperPoints.push_back(std::vector<c2t::Point>());
+    std::vector<c2t::Point> &curPts = clipperPoints.front();
     curPts.reserve(ppi.elementCount());
     for (int i = 0; i < ppi.elementCount(); ++i) {
         const QPainterPath::Element e = ppi.elementAt(i);
         if (e.isMoveTo() || i == ppi.elementCount() - 1
-                || (qAbs(e.x - curPts.front()->x) < 0.1
-                    && qAbs(e.y - curPts.front()->y) < 0.1)) {
+                || (qAbs(e.x - curPts.front().x) < 0.1
+                    && qAbs(e.y - curPts.front().y) < 0.1)) {
             if (curPts.size() > 2) {
-                p2t::CDT *cdt = new p2t::CDT(curPts);
-                cdt->Triangulate();
-                std::vector<p2t::Triangle*> tris = cdt->GetTriangles();
-                screenVertices_.reserve(screenVertices_.size() + int(tris.size()));
-                for (size_t i = 0; i < tris.size(); ++i) {
-                    p2t::Triangle *t = tris.at(i);
-                    for (int j = 0; j < 3; ++j) {
-                        p2t::Point *p = t->GetPoint(j);
-                        screenVertices_ << QPointF(p->x, p->y);
-                    }
+                c2t::clip2tri *cdt = new c2t::clip2tri();
+                std::vector<c2t::Point> outputTriangles;
+                cdt->triangulate(clipperPoints, outputTriangles, std::vector<c2t::Point>());
+                for (size_t i = 0; i < outputTriangles.size(); ++i) {
+                    screenVertices_ << QPointF(outputTriangles[i].x, outputTriangles[i].y);
                 }
                 delete cdt;
             }
             curPts.clear();
             curPts.reserve(ppi.elementCount() - i);
-            curPts.push_back(new p2t::Point(e.x, e.y));
+            curPts.push_back( c2t::Point(e.x, e.y));
         } else if (e.isLineTo()) {
-            curPts.push_back(new p2t::Point(e.x, e.y));
+            curPts.push_back( c2t::Point(e.x, e.y));
         } else {
             qWarning("Unhandled element type in polygon painterpath");
         }
     }
+#else // Old qTriangulate()-based code.
+    QTriangleSet ts = qTriangulate(ppi);
+    qreal *vx = ts.vertices.data();
 
-    if (curPts.size() > 0) {
-        qDeleteAll(curPts.begin(), curPts.end());
-        curPts.clear();
+    screenIndices_.reserve(ts.indices.size());
+    screenVertices_.reserve(ts.vertices.size());
+
+    if (ts.indices.type() == QVertexIndexVector::UnsignedInt) {
+        const quint32 *ix = reinterpret_cast<const quint32 *>(ts.indices.data());
+        for (int i = 0; i < (ts.indices.size()/3*3); ++i)
+            screenIndices_ << ix[i];
+    } else {
+        const quint16 *ix = reinterpret_cast<const quint16 *>(ts.indices.data());
+        for (int i = 0; i < (ts.indices.size()/3*3); ++i)
+            screenIndices_ << ix[i];
     }
+    for (int i = 0; i < (ts.vertices.size()/2*2); i += 2)
+        screenVertices_ << QPointF(vx[i], vx[i + 1]);
+#endif
 
     screenBounds_ = ppi.boundingRect();
-
 }
 
 QDeclarativePolygonMapItem::QDeclarativePolygonMapItem(QQuickItem *parent)
@@ -332,7 +342,7 @@ QDeclarativePolygonMapItem::QDeclarativePolygonMapItem(QQuickItem *parent)
 void QDeclarativePolygonMapItem::handleBorderUpdated()
 {
     borderGeometry_.markSourceDirty();
-    updateMapItem();
+    polishAndUpdate();
 }
 
 QDeclarativePolygonMapItem::~QDeclarativePolygonMapItem()
@@ -367,7 +377,7 @@ void QDeclarativePolygonMapItem::setMap(QDeclarativeGeoMap *quickMap, QGeoMap *m
     if (map) {
         geometry_.markSourceDirty();
         borderGeometry_.markSourceDirty();
-        updateMapItem();
+        polishAndUpdate();
     }
 }
 
@@ -420,10 +430,12 @@ void QDeclarativePolygonMapItem::setPath(const QJSValue &value)
         return;
 
     path_ = pathList;
-
+    geoLeftBound_ = QDeclarativePolylineMapItem::getLeftBound(path_, deltaXs_, minX_);
+    geometry_.setPreserveGeometry(true, geoLeftBound_);
+    borderGeometry_.setPreserveGeometry(true, geoLeftBound_);
     geometry_.markSourceDirty();
     borderGeometry_.markSourceDirty();
-    updateMapItem();
+    polishAndUpdate();
     emit pathChanged();
 }
 
@@ -438,41 +450,38 @@ void QDeclarativePolygonMapItem::setPath(const QJSValue &value)
 void QDeclarativePolygonMapItem::addCoordinate(const QGeoCoordinate &coordinate)
 {
     path_.append(coordinate);
-
+    geoLeftBound_ = QDeclarativePolylineMapItem::getLeftBound(path_, deltaXs_, minX_, geoLeftBound_);
+    geometry_.setPreserveGeometry(true, geoLeftBound_);
+    borderGeometry_.setPreserveGeometry(true, geoLeftBound_);
     geometry_.markSourceDirty();
     borderGeometry_.markSourceDirty();
-    updateMapItem();
+    polishAndUpdate();
     emit pathChanged();
 }
 
 /*!
     \qmlmethod void MapPolygon::removeCoordinate(coordinate)
 
-    Removes a coordinate from the path. If there are multiple instances of the
+    Removes \a coordinate from the path. If there are multiple instances of the
     same coordinate, the one added last is removed.
 
+    If \a coordinate is not in the path this method does nothing.
+
     \sa addCoordinate, path
-
 */
-
 void QDeclarativePolygonMapItem::removeCoordinate(const QGeoCoordinate &coordinate)
 {
     int index = path_.lastIndexOf(coordinate);
-
-    if (index == -1) {
-        qmlInfo(this) << COORD_NOT_BELONG_TO << QStringLiteral("PolygonMapItem");
+    if (index == -1)
         return;
-    }
 
-    if (path_.count() < index + 1) {
-        qmlInfo(this) << COORD_NOT_BELONG_TO << QStringLiteral("PolygonMapItem");
-        return;
-    }
     path_.removeAt(index);
-
+    geoLeftBound_ = QDeclarativePolylineMapItem::getLeftBound(path_, deltaXs_, minX_);
+    geometry_.setPreserveGeometry(true, geoLeftBound_);
+    borderGeometry_.setPreserveGeometry(true, geoLeftBound_);
     geometry_.markSourceDirty();
     borderGeometry_.markSourceDirty();
-    updateMapItem();
+    polishAndUpdate();
     emit pathChanged();
 }
 
@@ -496,7 +505,7 @@ void QDeclarativePolygonMapItem::setColor(const QColor &color)
 
     color_ = color;
     dirtyMaterial_ = true;
-    updateMapItem();
+    update();
     emit colorChanged(color_);
 }
 
@@ -526,7 +535,7 @@ QSGNode *QDeclarativePolygonMapItem::updateMapItemPaintNode(QSGNode *oldNode, Up
 /*!
     \internal
 */
-void QDeclarativePolygonMapItem::updateMapItem()
+void QDeclarativePolygonMapItem::updatePolish()
 {
     if (!map() || path_.count() == 0)
         return;
@@ -540,7 +549,7 @@ void QDeclarativePolygonMapItem::updateMapItem()
     QList<QGeoCoordinate> closedPath = path_;
     closedPath << closedPath.first();
     borderGeometry_.clear();
-    borderGeometry_.updateSourcePoints(*map(), closedPath);
+    borderGeometry_.updateSourcePoints(*map(), closedPath, geoLeftBound_);
 
     if (border_.color() != Qt::transparent && border_.width() > 0)
         borderGeometry_.updateScreenPoints(*map(), border_.width());
@@ -553,7 +562,6 @@ void QDeclarativePolygonMapItem::updateMapItem()
     setHeight(combined.height());
 
     setPositionOnMap(path_.at(0), -1 * geometry_.sourceBoundingBox().topLeft());
-    update();
 }
 
 /*!
@@ -589,7 +597,7 @@ void QDeclarativePolygonMapItem::afterViewportChanged(const QGeoMapViewportChang
     borderGeometry_.setPreserveGeometry(true, borderGeometry_.geoLeftBound());
     geometry_.markScreenDirty();
     borderGeometry_.markScreenDirty();
-    updateMapItem();
+    polishAndUpdate();
 }
 
 /*!
@@ -638,15 +646,13 @@ void QDeclarativePolygonMapItem::geometryChanged(const QRectF &newGeometry, cons
 
             path_.replace(i, coord);
         }
-
-        QGeoCoordinate leftBoundCoord = geometry_.geoLeftBound();
-        leftBoundCoord.setLongitude(QLocationUtils::wrapLong(leftBoundCoord.longitude()
-                           + newCoordinate.longitude() - firstLongitude));
-        geometry_.setPreserveGeometry(true, leftBoundCoord);
-        borderGeometry_.setPreserveGeometry(true, leftBoundCoord);
+        geoLeftBound_.setLongitude(QLocationUtils::wrapLong(geoLeftBound_.longitude()
+                                                            + newCoordinate.longitude() - firstLongitude));
+        geometry_.setPreserveGeometry(true, geoLeftBound_);
+        borderGeometry_.setPreserveGeometry(true, geoLeftBound_);
         geometry_.markSourceDirty();
         borderGeometry_.markSourceDirty();
-        updateMapItem();
+        polishAndUpdate();
         emit pathChanged();
     }
 
