@@ -37,6 +37,17 @@
 #ifndef QDECLARATIVEPOLYGONMAPITEM
 #define QDECLARATIVEPOLYGONMAPITEM
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  It exists purely as an
+// implementation detail.  This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
+
 #include "qdeclarativegeomapitembase_p.h"
 #include "qdeclarativepolylinemapitem_p.h"
 #include "qgeomapitemgeometry_p.h"
@@ -100,9 +111,9 @@ Q_SIGNALS:
 
 protected:
     void geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry) Q_DECL_OVERRIDE;
+    void updatePolish() Q_DECL_OVERRIDE;
 
 protected Q_SLOTS:
-    virtual void updateMapItem() Q_DECL_OVERRIDE;
     void handleBorderUpdated();
     virtual void afterViewportChanged(const QGeoMapViewportChangeEvent &event) Q_DECL_OVERRIDE;
 
@@ -111,11 +122,15 @@ private:
 
     QDeclarativeMapLineProperties border_;
     QList<QGeoCoordinate> path_;
+    QGeoCoordinate geoLeftBound_;
     QColor color_;
     bool dirtyMaterial_;
     QGeoMapPolygonGeometry geometry_;
     QGeoMapPolylineGeometry borderGeometry_;
     bool updatingGeometry_;
+    // for the left bound calculation
+    QVector<double> deltaXs_; // longitude deltas from path_[0]
+    double minX_;             // minimum value inside deltaXs_
 };
 
 //////////////////////////////////////////////////////////////////////
