@@ -1,31 +1,34 @@
 /****************************************************************************
  **
- ** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
- ** Contact: http://www.qt-project.org/legal
+ ** Copyright (C) 2015 The Qt Company Ltd.
+ ** Contact: http://www.qt.io/licensing/
  **
  ** This file is part of the QtLocation module of the Qt Toolkit.
  **
- ** $QT_BEGIN_LICENSE:LGPL21$
+ ** $QT_BEGIN_LICENSE:LGPL3$
  ** Commercial License Usage
  ** Licensees holding valid commercial Qt licenses may use this file in
  ** accordance with the commercial license agreement provided with the
  ** Software or, alternatively, in accordance with the terms contained in
- ** a written agreement between you and Digia. For licensing terms and
- ** conditions see http://qt.digia.com/licensing. For further information
- ** use the contact form at http://qt.digia.com/contact-us.
+ ** a written agreement between you and The Qt Company. For licensing terms
+ ** and conditions see http://www.qt.io/terms-conditions. For further
+ ** information use the contact form at http://www.qt.io/contact-us.
  **
  ** GNU Lesser General Public License Usage
  ** Alternatively, this file may be used under the terms of the GNU Lesser
- ** General Public License version 2.1 or version 3 as published by the Free
- ** Software Foundation and appearing in the file LICENSE.LGPLv21 and
- ** LICENSE.LGPLv3 included in the packaging of this file. Please review the
- ** following information to ensure the GNU Lesser General Public License
- ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
- ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+ ** General Public License version 3 as published by the Free Software
+ ** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+ ** packaging of this file. Please review the following information to
+ ** ensure the GNU Lesser General Public License version 3 requirements
+ ** will be met: https://www.gnu.org/licenses/lgpl.html.
  **
- ** In addition, as a special exception, Digia gives you certain additional
- ** rights. These rights are described in the Digia Qt LGPL Exception
- ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+ ** GNU General Public License Usage
+ ** Alternatively, this file may be used under the terms of the GNU
+ ** General Public License version 2.0 or later as published by the Free
+ ** Software Foundation and appearing in the file LICENSE.GPL included in
+ ** the packaging of this file. Please review the following information to
+ ** ensure the GNU General Public License version 2.0 requirements will be
+ ** met: http://www.gnu.org/licenses/gpl-2.0.html.
  **
  ** $QT_END_LICENSE$
  **
@@ -40,10 +43,7 @@
 
 #include <QtCore/QScopedValueRollback>
 #include <QtQml/QQmlInfo>
-#include <QtQml/QQmlContext>
 #include <QtQml/private/qqmlengine_p.h>
-#include <private/qqmlvaluetypewrapper_p.h>
-#include <private/qjsvalue_p.h>
 #include <QPainter>
 #include <QPainterPath>
 #include <QPainterPathStroker>
@@ -88,7 +88,7 @@ QT_BEGIN_NAMESPACE
     the Map grows in direct proportion to the number of points in the polyline.
 
     Like the other map objects, MapPolyline is normally drawn without a smooth
-    appearance. Setting the \l {QtQuick::Item::opacity}{opacity} property will force the object to
+    appearance. Setting the \l {Item::opacity}{opacity} property will force the object to
     be blended, which decreases performance considerably depending on the hardware in use.
 
     \note MapPolylines are implemented using the OpenGL GL_LINES
@@ -180,7 +180,8 @@ QGeoMapPolylineGeometry::QGeoMapPolylineGeometry()
     \internal
 */
 void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
-                                                 const QList<QGeoCoordinate> &path)
+                                                 const QList<QGeoCoordinate> &path,
+                                                 const QGeoCoordinate geoLeftBound)
 {
     bool foundValid = false;
     double minX = -1.0;
@@ -191,6 +192,8 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
     if (!sourceDirty_)
         return;
 
+    geoLeftBound_ = geoLeftBound;
+
     // clear the old data and reserve enough memory
     srcPoints_.clear();
     srcPoints_.reserve(path.size() * 2);
@@ -199,9 +202,10 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
 
     QDoubleVector2D origin, lastPoint, lastAddedPoint;
 
+    const double mapWidthHalf = map.width()/2.0;
     double unwrapBelowX = 0;
     if (preserveGeometry_)
-        unwrapBelowX = map.coordinateToScreenPosition(geoLeftBound_, false).x();
+        unwrapBelowX = map.coordinateToItemPosition(geoLeftBound_, false).x();
 
     for (int i = 0; i < path.size(); ++i) {
         const QGeoCoordinate &coord = path.at(i);
@@ -209,20 +213,30 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
         if (!coord.isValid())
             continue;
 
-        QDoubleVector2D point = map.coordinateToScreenPosition(coord, false);
+        QDoubleVector2D point = map.coordinateToItemPosition(coord, false);
 
         // We can get NaN if the map isn't set up correctly, or the projection
         // is faulty -- probably best thing to do is abort
         if (!qIsFinite(point.x()) || !qIsFinite(point.y()))
             return;
 
+        bool isPointLessThanUnwrapBelowX = (point.x() < unwrapBelowX);
+        bool isCoordNotLeftBound = !qFuzzyCompare(geoLeftBound_.longitude(), coord.longitude());
+        bool isPointNotUnwrapBelowX = !qFuzzyCompare(point.x(), unwrapBelowX);
+        bool isPointNotMapWidthHalf = !qFuzzyCompare(mapWidthHalf, point.x());
+
         // unwrap x to preserve geometry if moved to border of map
-        if (preserveGeometry_ && point.x() < unwrapBelowX && !qFuzzyCompare(point.x(), unwrapBelowX))
-            point.setX(unwrapBelowX + geoDistanceToScreenWidth(map, geoLeftBound_, coord));
+        if (preserveGeometry_ && isPointLessThanUnwrapBelowX
+                && isCoordNotLeftBound
+                && isPointNotUnwrapBelowX
+                && isPointNotMapWidthHalf) {
+            double distance = geoDistanceToScreenWidth(map, geoLeftBound_, coord);
+            point.setX(unwrapBelowX + distance);
+        }
 
         if (!foundValid) {
             foundValid = true;
-            srcOrigin_ = coord;
+            srcOrigin_ = coord;  // TODO: Make this consistent with the left bound
             origin = point;
             point = QDoubleVector2D(0,0);
 
@@ -254,8 +268,6 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
     }
 
     sourceBounds_ = QRectF(QPointF(minX, minY), QPointF(maxX, maxY));
-    geoLeftBound_ = map.screenPositionToCoordinate(
-                                    QDoubleVector2D(minX + origin.x(), minY + origin.y()), false);
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -381,7 +393,7 @@ void QGeoMapPolylineGeometry::updateScreenPoints(const QGeoMap &map,
     if (!screenDirty_)
         return;
 
-    QPointF origin = map.coordinateToScreenPosition(srcOrigin_, false).toPointF();
+    QPointF origin = map.coordinateToItemPosition(srcOrigin_, false).toPointF();
 
     if (!qIsFinite(origin.x()) || !qIsFinite(origin.y())) {
         clear();
@@ -419,24 +431,32 @@ void QGeoMapPolylineGeometry::updateScreenPoints(const QGeoMap &map,
     // not the number of vertices
     screenVertices_.reserve(ts.vertexCount());
 
-    screenOutline_ = QPainterPath();
+    QRectF bb;
 
-    QPolygonF tri;
+    QPointF pt;
     const float *vs = ts.vertices();
     for (int i = 0; i < (ts.vertexCount()/2*2); i += 2) {
-        screenVertices_ << QPointF(vs[i], vs[i + 1]);
+        pt = QPointF(vs[i], vs[i + 1]);
+        screenVertices_ << pt;
 
-        if (!qIsFinite(vs[i]) || !qIsFinite(vs[i + 1]))
+        if (!qIsFinite(pt.x()) || !qIsFinite(pt.y()))
             break;
 
-        tri << QPointF(vs[i], vs[i + 1]);
-        if (tri.size() == 4) {
-            tri.remove(0);
-            screenOutline_.addPolygon(tri);
+        if (!bb.contains(pt)) {
+            if (pt.x() < bb.left())
+                bb.setLeft(pt.x());
+
+            if (pt.x() > bb.right())
+                bb.setRight(pt.x());
+
+            if (pt.y() < bb.top())
+                bb.setTop(pt.y());
+
+            if (pt.y() > bb.bottom())
+                bb.setBottom(pt.y());
         }
     }
 
-    QRectF bb = screenOutline_.boundingRect();
     screenBounds_ = bb;
     this->translate( -1 * sourceBounds_.topLeft());
 }
@@ -462,7 +482,7 @@ void QDeclarativePolylineMapItem::updateAfterLinePropertiesChanged()
 {
     // mark dirty just in case we're a width change
     geometry_.markSourceDirty();
-    updateMapItem();
+    polishAndUpdate();
 }
 
 /*!
@@ -473,7 +493,7 @@ void QDeclarativePolylineMapItem::setMap(QDeclarativeGeoMap *quickMap, QGeoMap *
     QDeclarativeGeoMapItemBase::setMap(quickMap,map);
     if (map) {
         geometry_.markSourceDirty();
-        updateMapItem();
+        polishAndUpdate();
     }
 }
 
@@ -486,23 +506,22 @@ void QDeclarativePolylineMapItem::setMap(QDeclarativeGeoMap *quickMap, QGeoMap *
 
 QJSValue QDeclarativePolylineMapItem::path() const
 {
-    QQmlContext *context = QQmlEngine::contextForObject(parent());
+    QQmlContext *context = QQmlEngine::contextForObject(this);
+    if (!context)
+        return QJSValue();
     QQmlEngine *engine = context->engine();
-    QV8Engine *v8Engine = QQmlEnginePrivate::getV8Engine(engine);
-    QV4::ExecutionEngine *v4 = QV8Engine::getV4(v8Engine);
+    QV4::ExecutionEngine *v4 = QQmlEnginePrivate::getV4Engine(engine);
 
     QV4::Scope scope(v4);
     QV4::Scoped<QV4::ArrayObject> pathArray(scope, v4->newArrayObject(path_.length()));
     for (int i = 0; i < path_.length(); ++i) {
         const QGeoCoordinate &c = path_.at(i);
 
-        QQmlValueType *vt = QQmlValueTypeFactory::valueType(qMetaTypeId<QGeoCoordinate>());
-        QV4::ScopedValue cv(scope, QV4::QmlValueTypeWrapper::create(v8Engine, QVariant::fromValue(c), vt));
-
+        QV4::ScopedValue cv(scope, v4->fromVariant(QVariant::fromValue(c)));
         pathArray->putIndexed(i, cv);
     }
 
-    return new QJSValuePrivate(v4, QV4::ValueRef(pathArray));
+    return QJSValue(v4, pathArray.asReturnedValue());
 }
 
 void QDeclarativePolylineMapItem::setPath(const QJSValue &value)
@@ -524,59 +543,167 @@ void QDeclarativePolylineMapItem::setPath(const QJSValue &value)
         pathList.append(c);
     }
 
-    if (path_ == pathList)
+    setPathFromGeoList(pathList);
+}
+
+/*!
+    \internal
+*/
+void QDeclarativePolylineMapItem::setPathFromGeoList(const QList<QGeoCoordinate> &path)
+{
+    if (path_ == path)
         return;
 
-    path_ = pathList;
-
+    path_ = path;
+    geoLeftBound_ = getLeftBound(path_, deltaXs_, minX_);
+    geometry_.setPreserveGeometry(true, geoLeftBound_);
     geometry_.markSourceDirty();
-    updateMapItem();
+    polishAndUpdate();
     emit pathChanged();
 }
 
 /*!
-    \qmlmethod MapPolyline::addCoordinate(coordinate)
+    \qmlmethod int MapPolyline::pathLength()
 
-    Adds a coordinate to the path.
+    Returns the number of coordinates of the polyline.
 
-    \sa removeCoordinate, path
+    \since Qt Location 5.6
+
+    \sa path
 */
+int QDeclarativePolylineMapItem::pathLength() const
+{
+    return path_.size();
+}
 
+/*!
+    \qmlmethod void MapPolyline::addCoordinate(coordinate)
+
+    Adds a coordinate to the end of the path.
+
+    \sa insertCoordinate, removeCoordinate, path
+*/
 void QDeclarativePolylineMapItem::addCoordinate(const QGeoCoordinate &coordinate)
 {
     path_.append(coordinate);
-
+    geoLeftBound_ = getLeftBound(path_, deltaXs_, minX_, geoLeftBound_);
+    geometry_.setPreserveGeometry(true, geoLeftBound_);
     geometry_.markSourceDirty();
-    updateMapItem();
+    polishAndUpdate();
     emit pathChanged();
 }
 
 /*!
-    \qmlmethod MapPolyline::removeCoordinate(coordinate)
+    \qmlmethod void MapPolyline::insertCoordinate(index, coordinate)
 
-    Removes a coordinate from the path. If there are multiple instances of the
+    Inserts a \a coordinate to the path at the given \a index.
+
+    \since Qt Location 5.6
+
+    \sa addCoordinate, removeCoordinate, path
+*/
+void QDeclarativePolylineMapItem::insertCoordinate(int index, const QGeoCoordinate &coordinate)
+{
+    if (index < 0 || index > path_.size())
+        return;
+
+    path_.insert(index, coordinate);
+    geoLeftBound_ = getLeftBound(path_, deltaXs_, minX_);
+    geometry_.setPreserveGeometry(true, geoLeftBound_);
+    geometry_.markSourceDirty();
+    polishAndUpdate();
+    emit pathChanged();
+}
+
+/*!
+    \qmlmethod void MapPolyline::replaceCoordinate(index, coordinate)
+
+    Replaces the coordinate in the current path at the given \a index
+    with the new \a coordinate.
+
+    \since Qt Location 5.6
+
+    \sa addCoordinate, insertCoordinate, removeCoordinate, path
+*/
+void QDeclarativePolylineMapItem::replaceCoordinate(int index, const QGeoCoordinate &coordinate)
+{
+    if (index < 0 || index >= path_.size())
+        return;
+
+    path_[index] = coordinate;
+    geoLeftBound_ = getLeftBound(path_, deltaXs_, minX_);
+    geometry_.setPreserveGeometry(true, geoLeftBound_);
+    geometry_.markSourceDirty();
+    polishAndUpdate();
+    emit pathChanged();
+}
+
+/*!
+    \qmlmethod coordinate MapPolyline::coordinateAt(index)
+
+    Gets the coordinate of the polyline at the given \a index.
+    If the index is outside the path's bounds then an invalid
+    coordinate is returned.
+
+    \since Qt Location 5.6
+*/
+QGeoCoordinate QDeclarativePolylineMapItem::coordinateAt(int index) const
+{
+    if (index < 0 || index >= path_.size())
+        return QGeoCoordinate();
+
+    return path_.at(index);
+}
+
+/*!
+    \qmlmethod coordinate MapPolyline::containsCoordinate(coordinate)
+
+    Returns true if the given \a coordinate is part of the path.
+
+    \since Qt Location 5.6
+*/
+bool QDeclarativePolylineMapItem::containsCoordinate(const QGeoCoordinate &coordinate)
+{
+    return path_.indexOf(coordinate) > -1;
+}
+
+/*!
+    \qmlmethod void MapPolyline::removeCoordinate(coordinate)
+
+    Removes \a coordinate from the path. If there are multiple instances of the
     same coordinate, the one added last is removed.
 
-    \sa addCoordinate, path
-*/
+    If \a coordinate is not in the path this method does nothing.
 
+    \sa addCoordinate, insertCoordinate, path
+*/
 void QDeclarativePolylineMapItem::removeCoordinate(const QGeoCoordinate &coordinate)
 {
     int index = path_.lastIndexOf(coordinate);
+    removeCoordinate(index);
+}
 
-    if (index == -1) {
-        qmlInfo(this) << COORD_NOT_BELONG_TO << QStringLiteral("PolylineMapItem");
-        return;
-    }
+/*!
+    \qmlmethod void MapPolyline::removeCoordinate(index)
 
-    if (path_.count() < index + 1) {
-        qmlInfo(this) << COORD_NOT_BELONG_TO << QStringLiteral("PolylineMapItem");
+    Removes a coordinate from the path at the given \a index.
+
+    If \a index is invalid then this method does nothing.
+
+    \since Qt Location 5.6
+
+    \sa addCoordinate, insertCoordinate, path
+*/
+void QDeclarativePolylineMapItem::removeCoordinate(int index)
+{
+    if (index < 0 || index >= path_.size())
         return;
-    }
+
     path_.removeAt(index);
-
+    geoLeftBound_ = getLeftBound(path_, deltaXs_, minX_);
+    geometry_.setPreserveGeometry(true, geoLeftBound_);
     geometry_.markSourceDirty();
-    updateMapItem();
+    polishAndUpdate();
     emit pathChanged();
 }
 
@@ -599,6 +726,96 @@ QDeclarativeMapLineProperties *QDeclarativePolylineMapItem::line()
     return &line_;
 }
 
+QGeoCoordinate QDeclarativePolylineMapItem::computeLeftBound(const QList<QGeoCoordinate> &path,
+                                                             QVector<double> &deltaXs,
+                                                             double &minX)
+{
+    if (path.isEmpty()) {
+        minX = qInf();
+        return QGeoCoordinate();
+    }
+    deltaXs.resize(path.size());
+
+    deltaXs[0] = minX = 0.0;
+    int minId = 0;
+
+    for (int i = 1; i < path.size(); i++) {
+        const QGeoCoordinate &geoFrom = path.at(i-1);
+        const QGeoCoordinate &geoTo   = path.at(i);
+        double longiFrom    = geoFrom.longitude();
+        double longiTo      = geoTo.longitude();
+        double deltaLongi = longiTo - longiFrom;
+        if (qAbs(deltaLongi) > 180.0) {
+            if (longiTo > 0.0)
+                longiTo -= 360.0;
+            else
+                longiTo += 360.0;
+            deltaLongi =  longiTo - longiFrom;
+        }
+        deltaXs[i] = deltaXs[i-1] + deltaLongi;
+        if (deltaXs[i] < minX) {
+            minX = deltaXs[i];
+            minId = i;
+        }
+    }
+    return path.at(minId);
+}
+
+QGeoCoordinate QDeclarativePolylineMapItem::updateLeftBound(const QList<QGeoCoordinate> &path,
+                                                            QVector<double> &deltaXs,
+                                                            double &minX,
+                                                            QGeoCoordinate currentLeftBound)
+{
+    if (path.isEmpty()) {
+        deltaXs.clear();
+        minX = qInf();
+        return QGeoCoordinate();
+    } else if (path.size() == 1) {
+        deltaXs.resize(1);
+        deltaXs[0] = minX = 0.0;
+        return path.last();
+    } else if ( path.size() != deltaXs.size() + 1 ) {  // something went wrong. This case should not happen
+        return computeLeftBound(path, deltaXs, minX);
+    }
+
+    const QGeoCoordinate &geoFrom = path.at(path.size()-2);
+    const QGeoCoordinate &geoTo   = path.last();
+    double longiFrom    = geoFrom.longitude();
+    double longiTo      = geoTo.longitude();
+    double deltaLongi = longiTo - longiFrom;
+    if (qAbs(deltaLongi) > 180.0) {
+        if (longiTo > 0.0)
+            longiTo -= 360.0;
+        else
+            longiTo += 360.0;
+        deltaLongi =  longiTo - longiFrom;
+    }
+
+    deltaXs.push_back(deltaXs.last() + deltaLongi);
+    if (deltaXs.last() < minX) {
+        minX = deltaXs.last();
+        return path.last();
+    } else {
+        return currentLeftBound;
+    }
+}
+
+QGeoCoordinate QDeclarativePolylineMapItem::getLeftBound(const QList<QGeoCoordinate> &path,
+                                                         QVector<double> &deltaXs,
+                                                         double &minX)
+{
+    return QDeclarativePolylineMapItem::computeLeftBound(path, deltaXs, minX);
+}
+
+// Optimizing the common addCoordinate() path
+QGeoCoordinate QDeclarativePolylineMapItem::getLeftBound(const QList<QGeoCoordinate> &path,
+                                                         QVector<double> &deltaXs,
+                                                         double &minX,
+                                                         QGeoCoordinate currentLeftBound)
+{
+    return QDeclarativePolylineMapItem::updateLeftBound(path, deltaXs, minX, currentLeftBound);
+}
+
 /*!
     \internal
 */
@@ -610,7 +827,7 @@ void QDeclarativePolylineMapItem::geometryChanged(const QRectF &newGeometry, con
     }
 
     QDoubleVector2D newPoint = QDoubleVector2D(x(),y()) + QDoubleVector2D(geometry_.firstPointOffset());
-    QGeoCoordinate newCoordinate = map()->screenPositionToCoordinate(newPoint, false);
+    QGeoCoordinate newCoordinate = map()->itemPositionToCoordinate(newPoint, false);
     if (newCoordinate.isValid()) {
         double firstLongitude = path_.at(0).longitude();
         double firstLatitude = path_.at(0).latitude();
@@ -637,12 +854,11 @@ void QDeclarativePolylineMapItem::geometryChanged(const QRectF &newGeometry, con
             path_.replace(i, coord);
         }
 
-        QGeoCoordinate leftBoundCoord = geometry_.geoLeftBound();
-        leftBoundCoord.setLongitude(QLocationUtils::wrapLong(leftBoundCoord.longitude()
-                           + newCoordinate.longitude() - firstLongitude));
-        geometry_.setPreserveGeometry(true, leftBoundCoord);
+        geoLeftBound_.setLongitude(QLocationUtils::wrapLong(geoLeftBound_.longitude()
+                                                            + newCoordinate.longitude() - firstLongitude));
+        geometry_.setPreserveGeometry(true, geoLeftBound_);
         geometry_.markSourceDirty();
-        updateMapItem();
+        polishAndUpdate();
         emit pathChanged();
     }
 
@@ -678,13 +894,13 @@ void QDeclarativePolylineMapItem::afterViewportChanged(const QGeoMapViewportChan
     }
     geometry_.setPreserveGeometry(true, geometry_.geoLeftBound());
     geometry_.markScreenDirty();
-    updateMapItem();
+    polishAndUpdate();
 }
 
 /*!
     \internal
 */
-void QDeclarativePolylineMapItem::updateMapItem()
+void QDeclarativePolylineMapItem::updatePolish()
 {
     if (!map() || path_.count() == 0)
         return;
@@ -692,14 +908,13 @@ void QDeclarativePolylineMapItem::updateMapItem()
     QScopedValueRollback<bool> rollback(updatingGeometry_);
     updatingGeometry_ = true;
 
-    geometry_.updateSourcePoints(*map(), path_);
+    geometry_.updateSourcePoints(*map(), path_, geoLeftBound_);
     geometry_.updateScreenPoints(*map(), line_.width());
 
     setWidth(geometry_.sourceBoundingBox().width());
     setHeight(geometry_.sourceBoundingBox().height());
 
     setPositionOnMap(path_.at(0), -1 * geometry_.sourceBoundingBox().topLeft());
-    update();
 }
 
 /*!
@@ -716,17 +931,29 @@ QSGNode *QDeclarativePolylineMapItem::updateMapItemPaintNode(QSGNode *oldNode, U
     }
 
     //TODO: update only material
-    if (geometry_.isScreenDirty() || dirtyMaterial_) {
+    if (geometry_.isScreenDirty() || dirtyMaterial_ || !oldNode) {
         node->update(line_.color(), &geometry_);
         geometry_.setPreserveGeometry(false);
         geometry_.markClean();
+        dirtyMaterial_ = false;
     }
     return node;
 }
 
 bool QDeclarativePolylineMapItem::contains(const QPointF &point) const
 {
-    return geometry_.contains(point);
+    QVector<QPointF> vertices = geometry_.vertices();
+    QPolygonF tri;
+    for (int i = 0; i < vertices.size(); ++i) {
+        tri << vertices[i];
+        if (tri.size() == 3) {
+            if (tri.containsPoint(point,Qt::OddEvenFill))
+                return true;
+            tri.remove(0);
+        }
+    }
+
+    return false;
 }
 
 //////////////////////////////////////////////////////////////////////
