@@ -625,21 +625,20 @@ void QGeoServiceProviderPrivate::unload()
 /* Filter out any parameter that doesn't match any plugin */
 void QGeoServiceProviderPrivate::filterParameterMap()
 {
-    const QList<QString> availablePlugins =
-            QGeoServiceProviderPrivate::plugins().keys();
+    const auto availablePlugins = QGeoServiceProviderPrivate::plugins();
 
     cleanedParameterMap = parameterMap;
-    foreach (const QString& name, availablePlugins) {
-        if (name == providerName) // don't remove parameters for current provider
+    for (auto it = availablePlugins.keyBegin(), end = availablePlugins.keyEnd(); it != end; ++it) {
+        if (*it == providerName) // don't remove parameters for current provider
             continue;
 
         QVariantMap::iterator i = cleanedParameterMap.begin();
         while (i != cleanedParameterMap.end()) {
             // remove every parameter meant for other plugins
-            if (i.key().startsWith(QString(name + QLatin1Char('.'))))
+            if (i.key().startsWith(QString(*it + QLatin1Char('.'))))
                 i = cleanedParameterMap.erase(i);
             else
-                i++;
+                ++i;
         }
     }
 }
