@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtLocation of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -33,7 +39,6 @@
 
 package org.qtproject.qt5.android.positioning;
 
-import android.app.Activity;
 import android.content.Context;
 import android.location.GpsSatellite;
 import android.location.GpsStatus;
@@ -94,10 +99,10 @@ public class QtPositioning implements LocationListener
 
     private PositioningLooper looperThread;
 
-    static public void setActivity(Activity activity, Object activityDelegate)
+    static public void setContext(Context context)
     {
         try {
-            locationManager = (LocationManager) activity.getSystemService(Context.LOCATION_SERVICE);
+            locationManager = (LocationManager)context.getSystemService(Context.LOCATION_SERVICE);
         } catch(Exception e) {
             e.printStackTrace();
         }
@@ -173,6 +178,58 @@ public class QtPositioning implements LocationListener
         return false;
     }
 
+
+    static private void addActiveListener(QtPositioning listener, String provider)
+    {
+        int androidClassKey = listener.nativeClassReference;
+        //start update thread
+        listener.setActiveLooper(true);
+
+        if (runningListeners.containsKey(androidClassKey) && runningListeners.get(androidClassKey) != listener) {
+            removeActiveListener(androidClassKey);
+        }
+
+        locationManager.requestSingleUpdate(provider,
+                                            listener,
+                                            listener.looper());
+
+        runningListeners.put(androidClassKey, listener);
+    }
+
+
+    static private void addActiveListener(QtPositioning listener, String provider, long minTime, float minDistance)
+    {
+        int androidClassKey = listener.nativeClassReference;
+        //start update thread
+        listener.setActiveLooper(true);
+
+        if (runningListeners.containsKey(androidClassKey) && runningListeners.get(androidClassKey) != listener) {
+            removeActiveListener(androidClassKey);
+        }
+
+        locationManager.requestLocationUpdates(provider,
+                                               minTime, minDistance,
+                                               listener,
+                                               listener.looper());
+
+        runningListeners.put(androidClassKey, listener);
+    }
+
+
+    static private void removeActiveListener(QtPositioning listener)
+    {
+        removeActiveListener(listener.nativeClassReference);
+    }
+
+
+    static private void removeActiveListener(int androidClassKey)
+    {
+        QtPositioning listener = runningListeners.remove(androidClassKey);
+        locationManager.removeUpdates(listener);
+        listener.setActiveLooper(false);
+    }
+
+
     static public int startUpdates(int androidClassKey, int locationProvider, int updateInterval)
     {
         synchronized (m_syncObject) {
@@ -182,20 +239,17 @@ public class QtPositioning implements LocationListener
                 positioningListener.nativeClassReference = androidClassKey;
                 positioningListener.expectedProviders = locationProvider;
                 positioningListener.isSatelliteUpdate = false;
-                //start update thread
-                positioningListener.setActiveLooper(true);
 
                 if (updateInterval == 0)
-                    updateInterval = 1000; //don't update more often than once per second
+                    updateInterval = 50; //don't update more often than once per 50ms
 
                 positioningListener.updateIntervalTime = updateInterval;
                 if ((locationProvider & QT_GPS_PROVIDER) > 0) {
                     Log.d(TAG, "Regular updates using GPS " + updateInterval);
                     try {
-                        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
-                                                       updateInterval, 0,
-                                                       positioningListener,
-                                                       positioningListener.looper());
+                        addActiveListener(positioningListener,
+                                        LocationManager.GPS_PROVIDER,
+                                        updateInterval, 0);
                     } catch (SecurityException se) {
                         se.printStackTrace();
                         exceptionOccurred = true;
@@ -205,18 +259,16 @@ public class QtPositioning implements LocationListener
                 if ((locationProvider & QT_NETWORK_PROVIDER) > 0) {
                     Log.d(TAG, "Regular updates using network " + updateInterval);
                     try {
-                        locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
-                                                       updateInterval, 0,
-                                                       positioningListener,
-                                                       positioningListener.looper());
+                        addActiveListener(positioningListener,
+                                            LocationManager.NETWORK_PROVIDER,
+                                            updateInterval, 0);
                    } catch (SecurityException se) {
                        se.printStackTrace();
                        exceptionOccurred = true;
                    }
                 }
                 if (exceptionOccurred) {
-                    positioningListener.setActiveLooper(false);
-                    locationManager.removeUpdates(positioningListener);
+                    removeActiveListener(positioningListener);
                     return QT_ACCESS_ERROR;
                 }
 
@@ -225,7 +277,6 @@ public class QtPositioning implements LocationListener
                     return QT_CLOSED_ERROR;
                 }
 
-                runningListeners.put(androidClassKey, positioningListener);
             } catch(Exception e) {
                 e.printStackTrace();
                 return QT_POSITION_UNKNOWN_SOURCE_ERROR;
@@ -240,9 +291,7 @@ public class QtPositioning implements LocationListener
         synchronized (m_syncObject) {
             try {
                 Log.d(TAG, "Stopping updates");
-                QtPositioning listener = runningListeners.remove(androidClassKey);
-                locationManager.removeUpdates(listener);
-                listener.setActiveLooper(false);
+                removeActiveListener(androidClassKey);
             } catch(Exception e) {
                 e.printStackTrace();
                 return;
@@ -260,15 +309,11 @@ public class QtPositioning implements LocationListener
                 positioningListener.isSingleUpdate = true;
                 positioningListener.expectedProviders = locationProvider;
                 positioningListener.isSatelliteUpdate = false;
-                //start update thread
-                positioningListener.setActiveLooper(true);
 
                 if ((locationProvider & QT_GPS_PROVIDER) > 0) {
                     Log.d(TAG, "Single update using GPS");
                     try {
-                        locationManager.requestSingleUpdate(LocationManager.GPS_PROVIDER,
-                                                        positioningListener,
-                                                        positioningListener.looper());
+                        addActiveListener(positioningListener, LocationManager.GPS_PROVIDER);
                     } catch (SecurityException se) {
                         se.printStackTrace();
                         exceptionOccurred = true;
@@ -278,17 +323,14 @@ public class QtPositioning implements LocationListener
                 if ((locationProvider & QT_NETWORK_PROVIDER) > 0) {
                     Log.d(TAG, "Single update using network");
                     try {
-                        locationManager.requestSingleUpdate(LocationManager.NETWORK_PROVIDER,
-                                                         positioningListener,
-                                                         positioningListener.looper());
+                        addActiveListener(positioningListener, LocationManager.NETWORK_PROVIDER);
                     } catch (SecurityException se) {
                          se.printStackTrace();
                          exceptionOccurred = true;
                     }
                 }
                 if (exceptionOccurred) {
-                    positioningListener.setActiveLooper(false);
-                    locationManager.removeUpdates(positioningListener);
+                    removeActiveListener(positioningListener);
                     return QT_ACCESS_ERROR;
                 }
 
@@ -298,7 +340,6 @@ public class QtPositioning implements LocationListener
                     return QT_CLOSED_ERROR;
                 }
 
-                runningListeners.put(androidClassKey, positioningListener);
             } catch(Exception e) {
                 e.printStackTrace();
                 return QT_POSITION_UNKNOWN_SOURCE_ERROR;
@@ -318,29 +359,24 @@ public class QtPositioning implements LocationListener
                 positioningListener.nativeClassReference = androidClassKey;
                 positioningListener.expectedProviders = 1; //always satellite provider
                 positioningListener.isSingleUpdate = isSingleRequest;
-                //start update thread
-                positioningListener.setActiveLooper(true);
 
                 if (updateInterval == 0)
-                    updateInterval = 1000; //don't update more often than once per second
+                    updateInterval = 50; //don't update more often than once per 50ms
 
                 if (isSingleRequest)
                     Log.d(TAG, "Single update for Satellites " + updateInterval);
                 else
                     Log.d(TAG, "Regular updates for Satellites " + updateInterval);
                 try {
-                    locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
-                                               updateInterval, 0,
-                                               positioningListener,
-                                               positioningListener.looper());
+                    addActiveListener(positioningListener, LocationManager.GPS_PROVIDER,
+                                        updateInterval, 0);
                 } catch (SecurityException se) {
                     se.printStackTrace();
                     exceptionOccurred = true;
                 }
 
                 if (exceptionOccurred) {
-                    positioningListener.setActiveLooper(false);
-                    locationManager.removeUpdates(positioningListener);
+                    removeActiveListener(positioningListener);
                     return QT_ACCESS_ERROR;
                 }
 
@@ -350,7 +386,6 @@ public class QtPositioning implements LocationListener
                     return QT_CLOSED_ERROR;
                 }
 
-                runningListeners.put(androidClassKey, positioningListener);
             } catch(Exception e) {
                 e.printStackTrace();
                 return QT_SATELLITE_UNKNOWN_SOURCE_ERROR;
