@@ -39,7 +39,9 @@
 #include <QCoreApplication>
 #include <QMutex>
 #include <qfunctions_winrt.h>
+#ifdef Q_OS_WINRT
 #include <private/qeventdispatcher_winrt_p.h>
+#endif
 
 #include <functional>
 #include <windows.system.h>
@@ -64,6 +66,16 @@ QT_BEGIN_NAMESPACE
 
 Q_DECLARE_METATYPE(QGeoPositionInfo)
 
+#ifndef Q_OS_WINRT
+namespace QEventDispatcherWinRT {
+HRESULT runOnXamlThread(const std::function<HRESULT ()> &delegate, bool waitForRun = true)
+{
+    Q_UNUSED(waitForRun);
+    return delegate();
+}
+}
+#endif
+
 class QGeoPositionInfoSourceWinRTPrivate {
 public:
     ComPtr<IGeolocator> locator;
@@ -331,7 +343,7 @@ void QGeoPositionInfoSourceWinRT::virtualPositionUpdate()
     // We can only do this if we received a valid position before
     if (d->lastPosition.isValid()) {
         QGeoPositionInfo sent = d->lastPosition;
-        sent.setTimestamp(sent.timestamp().addMSecs(updateInterval()));
+        sent.setTimestamp(QDateTime::currentDateTime());
         d->lastPosition = sent;
         emit positionUpdated(sent);
     }
@@ -474,24 +486,7 @@ HRESULT QGeoPositionInfoSourceWinRT::onPositionChanged(IGeolocator *locator, IPo
         currentInfo.setAttribute(QGeoPositionInfo::GroundSpeed, value);
     }
 
-    DateTime dateTime;
-    hr = coord->get_Timestamp(&dateTime);
-
-    if (dateTime.UniversalTime > 0) {
-        ULARGE_INTEGER uLarge;
-        uLarge.QuadPart = dateTime.UniversalTime;
-        FILETIME fileTime;
-        fileTime.dwHighDateTime = uLarge.HighPart;
-        fileTime.dwLowDateTime = uLarge.LowPart;
-        SYSTEMTIME systemTime;
-        if (FileTimeToSystemTime(&fileTime, &systemTime)) {
-            currentInfo.setTimestamp(QDateTime(QDate(systemTime.wYear, systemTime.wMonth,
-                                                     systemTime.wDay),
-                                               QTime(systemTime.wHour, systemTime.wMinute,
-                                                     systemTime.wSecond, systemTime.wMilliseconds),
-                                               Qt::UTC));
-        }
-    }
+    currentInfo.setTimestamp(QDateTime::currentDateTime());
 
     emit nativePositionUpdate(currentInfo);
 
@@ -507,7 +502,7 @@ HRESULT QGeoPositionInfoSourceWinRT::onStatusChanged(IGeolocator*, IStatusChange
 
 bool QGeoPositionInfoSourceWinRT::requestAccess() const
 {
-#if _MSC_VER >= 1900
+#if _MSC_VER >= 1900 && defined(Q_OS_WINRT)
     static GeolocationAccessStatus accessStatus = GeolocationAccessStatus_Unspecified;
     static ComPtr<IGeolocatorStatics> statics;
 
