@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -62,8 +57,6 @@ public:
         capabilities.setMinimumZoomLevel(0.0);
         capabilities.setMaximumZoomLevel(20.0);
         capabilities.setSupportsBearing(true);
-        setCameraCapabilities(capabilities);
-
         setTileSize(QSize(256, 256));
 
         QList<QGeoMapType> mapTypes;
@@ -73,14 +66,25 @@ public:
         setSupportedMapTypes(mapTypes);
 
         QGeoTileFetcherTest *fetcher = new QGeoTileFetcherTest(this);
-        fetcher->setParams(parameters);
-        fetcher->setTileSize(QSize(256, 255));
+        if (parameters.contains(QStringLiteral("finishRequestImmediately")))
+            fetcher->setFinishRequestImmediately(parameters.value(QStringLiteral("finishRequestImmediately")).toBool());
+        if (parameters.contains(QStringLiteral("tileSize"))) {
+            int tileSize = parameters.value(QStringLiteral("tileSize")).toInt();
+            setTileSize(QSize(tileSize, tileSize));
+        }
+        if (parameters.contains(QStringLiteral("maxZoomLevel"))) {
+            double maxZoomLevel = parameters.value(QStringLiteral("maxZoomLevel")).toDouble();
+            capabilities.setMaximumZoomLevel(maxZoomLevel);
+        }
+
+        setCameraCapabilities(capabilities);
+        fetcher->setTileSize(tileSize());
         setTileFetcher(fetcher);
     }
 
     QGeoMap *createMap()
     {
-        return new QGeoTiledMapTest(this);;
+        return new QGeoTiledMapTest(this);
     }
 
 };
