@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -90,7 +77,7 @@ private:
                       QList<QPlaceReply::Error> *errors,
                       QList<ExpectedResults> *results);
 
-    static const QLatin1String AuvergneEmbassyId;
+    static const QLatin1String ValidKnownPlaceId;
     static const QLatin1String ProxyEnv;
     static const QLatin1String AppIdEnv;
     static const QLatin1String TokenEnv;
@@ -100,10 +87,13 @@ private:
 
 Q_DECLARE_METATYPE(tst_QPlaceManagerNokia::ExpectedResults)
 
-const QLatin1String tst_QPlaceManagerNokia::AuvergneEmbassyId("250u09tv-be16478e55314b338c551aab2651c9d3");
+// ValidKnownPlaceId is the id of a place with a full complement of place content. Editorials,
+// reviews, images, recommendations. If it disappears these tests will fail.
+// Currently it is set to an Eiffel Tower tourist office.
+const QLatin1String tst_QPlaceManagerNokia::ValidKnownPlaceId("250u09tu-4561b8da952f4fd79c4e1998c3fcf032");
 
 const QLatin1String tst_QPlaceManagerNokia::ProxyEnv("NOKIA_PLUGIN_PROXY");
-const QLatin1String tst_QPlaceManagerNokia::AppIdEnv("NOKIA_APP_ID");
+const QLatin1String tst_QPlaceManagerNokia::AppIdEnv("NOKIA_APPID");
 const QLatin1String tst_QPlaceManagerNokia::TokenEnv("NOKIA_TOKEN");
 
 tst_QPlaceManagerNokia::tst_QPlaceManagerNokia()
@@ -114,19 +104,22 @@ void tst_QPlaceManagerNokia::initTestCase()
 {
     QVariantMap params;
     QStringList providers = QGeoServiceProvider::availableServiceProviders();
-    QVERIFY(providers.contains("nokia"));
+    QVERIFY(providers.contains("here"));
+#ifndef QT_NO_PROCESS
     QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
 
     if (!(env.contains(AppIdEnv) && env.contains(TokenEnv)))
-        QSKIP("NOKIA_APP_ID and NOKIA_TOKEN environment variables not set");\
+        QSKIP("NOKIA_APPID and NOKIA_TOKEN environment variables not set");\
 
-    params.insert(QStringLiteral("app_id"), env.value(AppIdEnv));
-    params.insert(QStringLiteral("token"), env.value(TokenEnv));
+    params.insert(QStringLiteral("here.app_id"), env.value(AppIdEnv));
+    params.insert(QStringLiteral("here.token"), env.value(TokenEnv));
 
     if (env.contains(ProxyEnv))
-        params.insert(QStringLiteral("proxy"), env.value(ProxyEnv));
-
-    provider = new QGeoServiceProvider("nokia", params);
+        params.insert(QStringLiteral("here.proxy"), env.value(ProxyEnv));
+#else
+    QSKIP("Cannot parse process environment, NOKIA_APPID and NOKIA_TOKEN not set");
+#endif
+    provider = new QGeoServiceProvider("here", params);
     placeManager = provider->placeManager();
     QVERIFY(placeManager);
 }
@@ -335,7 +328,7 @@ void tst_QPlaceManagerNokia::recommendations_data()
     eatDrinkCat.setCategoryId(QStringLiteral("eat-drink"));
 
     QTest::newRow("search recommendations with valid id")
-            << QString(AuvergneEmbassyId)
+            << QString(ValidKnownPlaceId)
             << QString()
             << QGeoShape()
             << QList<QPlaceCategory>()
@@ -349,28 +342,28 @@ void tst_QPlaceManagerNokia::recommendations_data()
             << QPlaceReply::PlaceDoesNotExistError;
 
     QTest::newRow("search for recommendations with id and search term")
-            << QString(AuvergneEmbassyId)
+            << QString(ValidKnownPlaceId)
             << QStringLiteral("sushi")
             << QGeoShape()
             << QList<QPlaceCategory>()
             << QPlaceReply::BadArgumentError;
 
     QTest::newRow("search for recommendations with an id and category")
-            << QString(AuvergneEmbassyId)
+            << QString(ValidKnownPlaceId)
             << QString()
             << QGeoShape()
             << (QList<QPlaceCategory>() << eatDrinkCat)
             << QPlaceReply::BadArgumentError;
 
     QTest::newRow("search for recommendations with id, search term and category")
-            << QString(AuvergneEmbassyId)
+            << QString(ValidKnownPlaceId)
             << QStringLiteral("sushi")
             << QGeoShape()
             << (QList<QPlaceCategory>() << eatDrinkCat)
             << QPlaceReply::BadArgumentError;
 
     QTest::newRow("search for recommendations with an id and search area")
-            << QString(AuvergneEmbassyId)
+            << QString(ValidKnownPlaceId)
             << QString()
             << static_cast<QGeoShape>(QGeoCircle(QGeoCoordinate(-27.5, 153)))
             << QList<QPlaceCategory>()
@@ -379,9 +372,10 @@ void tst_QPlaceManagerNokia::recommendations_data()
 
 void tst_QPlaceManagerNokia::details()
 {
+    QSKIP("Fetching details from HERE place server always fails - QTBUG-44837");
     //fetch the details of a valid place
     QPlace place;
-    QVERIFY(doFetchDetails(AuvergneEmbassyId, &place));
+    QVERIFY(doFetchDetails(ValidKnownPlaceId, &place));
     QVERIFY(!place.name().isEmpty());
     QVERIFY(!place.icon().url().isEmpty());
     QStringList contactTypes = place.contactTypes();
@@ -437,10 +431,17 @@ void tst_QPlaceManagerNokia::categories()
         //check we can retrieve the very same category by id
         QCOMPARE(placeManager->category(category.categoryId()), category);
 
-        //since the nokia plugin only supports a single level category tree
-        //check that there are no parent or children categories
+        //the here plugin supports a two-level level category tree
         QVERIFY(placeManager->parentCategoryId(category.categoryId()).isEmpty());
-        QVERIFY(placeManager->childCategories(category.categoryId()).isEmpty());
+        const QList<QPlaceCategory> childCats =
+                placeManager->childCategories(category.categoryId());
+        if (!childCats.isEmpty()) {
+            foreach (const QPlaceCategory &child, childCats) {
+                // only two levels of categories hence 2.nd level has no further children
+                QVERIFY(placeManager->childCategories(child.categoryId()).isEmpty());
+                QVERIFY(placeManager->parentCategoryId(child.categoryId()) == category.categoryId());
+            }
+        }
     }
 }
 
@@ -515,17 +516,9 @@ void tst_QPlaceManagerNokia::suggestions_data()
 
 void tst_QPlaceManagerNokia::suggestionsMisc()
 {
-    //check providing an offset
+    //check providing a distance relevancy hint (should be ignored)
     QPlaceSearchRequest searchRequest;
-    searchRequest.setSearchArea(QGeoCircle(QGeoCoordinate(-27.5, 153)));
-    searchRequest.setSearchTerm(QStringLiteral("sus"));
-    searchRequest.setOffset(5);
     QStringList results;
-    QVERIFY(doSearchSuggestions(searchRequest, &results, QPlaceReply::BadArgumentError));
-    QCOMPARE(results.count(), 0);
-    searchRequest.clear();
-
-    //check porviding a distance relevancy hint (should be ignored)
     searchRequest.setSearchArea(QGeoCircle(QGeoCoordinate(-27.5, 153)));
     searchRequest.setSearchTerm(QStringLiteral("sus"));
     searchRequest.setRelevanceHint(QPlaceSearchRequest::DistanceHint);
@@ -583,14 +576,21 @@ void tst_QPlaceManagerNokia::locale()
         }
     }
 
+    // we are skipping the check below because we are requesting
+    // details for a place without a search before. This implies
+    // URL templating must be possible which the HERE place
+    // server refuses.
+
+    QSKIP("remainder of test skipped due to QTBUG-44837");
+
     //check that setting a locale will affect place detail fetches.
     QPlace place;
     placeManager->setLocale(QLocale("en"));
-    QVERIFY(doFetchDetails(AuvergneEmbassyId,
+    QVERIFY(doFetchDetails(ValidKnownPlaceId,
                            &place));
     QString englishName = place.name();
     placeManager->setLocale(QLocale("fr"));
-    QVERIFY(doFetchDetails(AuvergneEmbassyId,
+    QVERIFY(doFetchDetails(ValidKnownPlaceId,
                            &place));
     QVERIFY(englishName != place.name());
 }
@@ -602,9 +602,9 @@ void tst_QPlaceManagerNokia::content()
     //check fetching of content
     QPlaceContentRequest request;
     request.setContentType(type);
+    request.setPlaceId(ValidKnownPlaceId);
     QPlaceContent::Collection results;
-    QVERIFY(doFetchContent(AuvergneEmbassyId,
-                            request, &results));
+    QVERIFY(doFetchContent(request, &results));
 
     QVERIFY(results.count() > 0);
 
@@ -638,39 +638,18 @@ void tst_QPlaceManagerNokia::content()
     }
 
     //check total count
-    QPlaceContentReply *contentReply =
-            placeManager->getPlaceContent(AuvergneEmbassyId,
-                                          request);
+    QPlaceContentReply *contentReply = placeManager->getPlaceContent(request);
     QSignalSpy contentSpy(contentReply, SIGNAL(finished()));
     QTRY_VERIFY_WITH_TIMEOUT(contentSpy.count() ==1, Timeout);
     QVERIFY(contentReply->totalCount() > 0);
 
     if (contentReply->totalCount() >= 2) {
-        //try testing with an offset
-        request.setOffset(1);
-        QPlaceContent::Collection newResults;
-        QVERIFY(doFetchContent(AuvergneEmbassyId,
-                                request, &newResults));
-        QVERIFY(!newResults.keys().contains(0));
-        QCOMPARE(newResults.value(1), results.value(1));
-
         //try testing with a limit
-        request.setOffset(0);
         request.setLimit(1);
-        QVERIFY(doFetchContent(AuvergneEmbassyId,
-                                request, &newResults));
+        QPlaceContent::Collection newResults;
+        QVERIFY(doFetchContent(request, &newResults));
         QCOMPARE(newResults.count(), 1);
         QCOMPARE(newResults.values().first(), results.value(0));
-
-        //try testing both limit and offset
-        if (contentReply->totalCount()  >= 3) {
-            request.setLimit(1);
-            request.setOffset(1);
-            QVERIFY(doFetchContent(AuvergneEmbassyId,
-                                    request, &newResults));
-            QCOMPARE(newResults.count(), 1);
-            QCOMPARE(newResults.values().first(), results.value(1));
-        }
     }
 }
 
@@ -686,13 +665,13 @@ void tst_QPlaceManagerNokia::content_data()
 void tst_QPlaceManagerNokia::unsupportedFunctions()
 {
     QPlace place;
-    place.setName(QLatin1String("Brisbane"));
+    place.setName(QStringLiteral("Brisbane"));
 
     QVERIFY(doSavePlace(place, QPlaceReply::UnsupportedError));
     QVERIFY(doRemovePlace(place, QPlaceReply::UnsupportedError));
 
     QPlaceCategory category;
-    category.setName(QLatin1String("Accommodation"));
+    category.setName(QStringLiteral("Accommodation"));
     QVERIFY(doSaveCategory(category, QPlaceReply::UnsupportedError));
     QVERIFY(doRemoveCategory(category, QPlaceReply::UnsupportedError));
 }
